<?php

namespace Drupal\layout_builder_kit\Controller;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Url;
use Drupal\layout_builder_kit\Entity\LBKIconInterface;

/**
 * Class LBKIconController.
 *
 *  Returns responses for LBKIcon routes.
 */
class LBKIconController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * Displays a LBKIcon  revision.
   *
   * @param int $lbk_icon_revision
   *   The LBKIcon  revision ID.
   *
   * @return array
   *   An array suitable for drupal_render().
   */
  public function revisionShow($lbk_icon_revision) {
    $lbk_icon = $this->entityManager()->getStorage('lbk_icon')->loadRevision($lbk_icon_revision);
    $view_builder = $this->entityManager()->getViewBuilder('lbk_icon');

    return $view_builder->view($lbk_icon);
  }

  /**
   * Page title callback for a LBKIcon  revision.
   *
   * @param int $lbk_icon_revision
   *   The LBKIcon  revision ID.
   *
   * @return string
   *   The page title.
   */
  public function revisionPageTitle($lbk_icon_revision) {
    $lbk_icon = $this->entityManager()->getStorage('lbk_icon')->loadRevision($lbk_icon_revision);
    return $this->t('Revision of %title from %date', ['%title' => $lbk_icon->label(), '%date' => format_date($lbk_icon->getRevisionCreationTime())]);
  }

  /**
   * Generates an overview table of older revisions of a LBKIcon .
   *
   * @param \Drupal\layout_builder_kit\Entity\LBKIconInterface $lbk_icon
   *   A LBKIcon  object.
   *
   * @return array
   *   An array as expected by drupal_render().
   */
  public function revisionOverview(LBKIconInterface $lbk_icon) {
    $account = $this->currentUser();
    $langcode = $lbk_icon->language()->getId();
    $langname = $lbk_icon->language()->getName();
    $languages = $lbk_icon->getTranslationLanguages();
    $has_translations = (count($languages) > 1);
    $lbk_icon_storage = $this->entityManager()->getStorage('lbk_icon');

    $build['#title'] = $has_translations ? $this->t('@langname revisions for %title', ['@langname' => $langname, '%title' => $lbk_icon->label()]) : $this->t('Revisions for %title', ['%title' => $lbk_icon->label()]);
    $header = [$this->t('Revision'), $this->t('Operations')];

    $revert_permission = (($account->hasPermission("revert all lbkicon revisions") || $account->hasPermission('administer lbkicon entities')));
    $delete_permission = (($account->hasPermission("delete all lbkicon revisions") || $account->hasPermission('administer lbkicon entities')));

    $rows = [];

    $vids = $lbk_icon_storage->revisionIds($lbk_icon);

    $latest_revision = TRUE;

    foreach (array_reverse($vids) as $vid) {
      /** @var \Drupal\layout_builder_kit\LBKIconInterface $revision */
      $revision = $lbk_icon_storage->loadRevision($vid);
      // Only show revisions that are affected by the language that is being
      // displayed.
      if ($revision->hasTranslation($langcode) && $revision->getTranslation($langcode)->isRevisionTranslationAffected()) {
        $username = [
          '#theme' => 'username',
          '#account' => $revision->getRevisionUser(),
        ];

        // Use revision link to link to revisions that are not active.
        $date = \Drupal::service('date.formatter')->format($revision->getRevisionCreationTime(), 'short');
        if ($vid != $lbk_icon->getRevisionId()) {
          $link = $this->l($date, new Url('entity.lbk_icon.revision', ['lbk_icon' => $lbk_icon->id(), 'lbk_icon_revision' => $vid]));
        }
        else {
          $link = $lbk_icon->link($date);
        }

        $row = [];
        $column = [
          'data' => [
            '#type' => 'inline_template',
            '#template' => '{% trans %}{{ date }} by {{ username }}{% endtrans %}{% if message %}<p class="revision-log">{{ message }}</p>{% endif %}',
            '#context' => [
              'date' => $link,
              'username' => \Drupal::service('renderer')->renderPlain($username),
              'message' => ['#markup' => $revision->getRevisionLogMessage(), '#allowed_tags' => Xss::getHtmlTagList()],
            ],
          ],
        ];
        $row[] = $column;

        if ($latest_revision) {
          $row[] = [
            'data' => [
              '#prefix' => '<em>',
              '#markup' => $this->t('Current revision'),
              '#suffix' => '</em>',
            ],
          ];
          foreach ($row as &$current) {
            $current['class'] = ['revision-current'];
          }
          $latest_revision = FALSE;
        }
        else {
          $links = [];
          if ($revert_permission) {
            $links['revert'] = [
              'title' => $this->t('Revert'),
              'url' => $has_translations ?
              Url::fromRoute('entity.lbk_icon.translation_revert', ['lbk_icon' => $lbk_icon->id(), 'lbk_icon_revision' => $vid, 'langcode' => $langcode]) :
              Url::fromRoute('entity.lbk_icon.revision_revert', ['lbk_icon' => $lbk_icon->id(), 'lbk_icon_revision' => $vid]),
            ];
          }

          if ($delete_permission) {
            $links['delete'] = [
              'title' => $this->t('Delete'),
              'url' => Url::fromRoute('entity.lbk_icon.revision_delete', ['lbk_icon' => $lbk_icon->id(), 'lbk_icon_revision' => $vid]),
            ];
          }

          $row[] = [
            'data' => [
              '#type' => 'operations',
              '#links' => $links,
            ],
          ];
        }

        $rows[] = $row;
      }
    }

    $build['lbk_icon_revisions_table'] = [
      '#theme' => 'table',
      '#rows' => $rows,
      '#header' => $header,
    ];

    return $build;
  }

}
