<?php

namespace Drupal\layout_builder_kit\Plugin\Block\LBKBookNavigation;

use Drupal\book\BookOutlineStorageInterface;
use Drupal\Core\Config\ConfigManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\Plugin\DataType\EntityAdapter;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\Context\EntityContext;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\layout_builder\Plugin\SectionStorage\OverridesSectionStorage;
use Drupal\layout_builder_kit\LinkProviderInterface;
use Drupal\layout_builder_kit\Plugin\Block\LBKBaseComponent;
use Drupal\node\Entity\Node;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'LBKBookNavigation' block.
 *
 * @Block(
 *  id = "lbk_book_navigation",
 *  admin_label = @Translation("Book Navigation (LBK)"),
 * )
 */
class LBKBookNavigation extends LBKBaseComponent implements ContainerFactoryPluginInterface {

  /**
   * Drupal\Core\Routing\CurrentRouteMatch class.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  private $currentRouteMatch;

  /**
   * Drupal\book\BookOutlineStorageInterface definition.
   *
   * @var \Drupal\book\BookOutlineStorageInterface
   */
  private $bookOutlineStorage;

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private $entityTypeManager;

  /**
   * Drupal\Core\Config\ConfigManagerInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigManagerInterface
   */
  protected $configManager;

  /**
   * Drupal\layout_builder_kit\LinkProviderInterface definition.
   *
   * @var \Drupal\layout_builder_kit\LinkProviderInterface
   */
  private $linkProvider;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $pluginId, $pluginDefinition) {
    return new static(
        $configuration,
        $pluginId,
        $pluginDefinition,
        $container->get('entity_type.manager'),
        $container->get('config.manager'),
        $container->get('current_route_match'),
        $container->get("book.outline_storage"),
        $container->get("layout_builder_kit.link_provider")
    );
  }

  /**
   * Constructs a new LBKBookNavigationComponent object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $pluginId
   *   The plugin_id for the plugin instance.
   * @param mixed $pluginDefinition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The EntityTypeManagerInterface service.
   * @param \Drupal\Core\Config\ConfigManagerInterface $configManager
   *   The ConfigManagerInterface service.
   * @param \Drupal\Core\Routing\CurrentRouteMatch $currentRouteMatch
   *   The CurrentRouteMatch service.
   * @param \Drupal\book\BookOutlineStorageInterface $bookOutlineStorage
   *   The BookOutlineStorageInterface service.
   * @param \Drupal\layout_builder_kit\LinkProviderInterface $linkProvider
   *   The LinkProviderInterface service.
   */
  public function __construct(array $configuration,
                              $pluginId,
                              $pluginDefinition,
                              EntityTypeManagerInterface $entityTypeManager,
                              ConfigManagerInterface $configManager,
                              CurrentRouteMatch $currentRouteMatch,
                              BookOutlineStorageInterface $bookOutlineStorage,
                              LinkProviderInterface $linkProvider
  ) {
    parent::__construct($configuration, $pluginId, $pluginDefinition);
    $this->entityTypeManager = $entityTypeManager;
    $this->currentRouteMatch = $currentRouteMatch;
    $this->configManager = $configManager;
    $this->bookOutlineStorage = $bookOutlineStorage;
    $this->entityTypeManager = $entityTypeManager;
    $this->linkProvider = $linkProvider;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [

    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $formState) {

    $form['#attached']['library'] = ['layout_builder_kit/book-navigation-styling'];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $formState) {}

  /**
   * {@inheritdoc}
   */
  public function build() {

    $content = $this->nodeObjects();

    if ($content) {
      $build = [];
      // Add template.
      $build['#theme'] = 'LBKBookNavigation';
      // Attach library.
      $build['#attached']['library'] = ['layout_builder_kit/book-navigation-styling'];
      $build['#title'] = $this->configuration['title'];
      $build['#display_title'] = $this->configuration['display_title'];
      $build['#classes'] = $this->configuration['classes'];
      $build['#content'] = $content;

      return $build;
    }
  }

  /**
   * Get Books List of Node.
   *
   * @return array
   *   Return array with Content Books List.
   */
  public function nodeObjects() {

    $node = $this->getNodeObject();
    $contentNode = [];

    if ($node instanceof Node) {
      $nid = $node->id();
      $book = $this->bookOutlineStorage->getBooks([$nid]);
      $book = $this->bookOutlineStorage
        ->loadBookChildren($book[0]);

      $nidParentActive = 0;
      foreach ($book as $keyParent => $data) {
        $bookChild = $this->bookOutlineStorage
          ->loadBookChildren($keyParent);
        if ($keyParent == $nid) {
          $nidParentActive = $keyParent;
        }
        foreach ($bookChild as $key => $child) {
          if ($key == $nid) {
            $nidParentActive = $keyParent;
          }
        }
      }

      $node_parent = $this->entityTypeManager
        ->getStorage('node')
        ->load($nidParentActive);
      $titleKeyParent = '';
      if ($node_parent) {
        $titleKeyParent = $node_parent->title->value;
      }

      $contentNode['parent_title'] = $titleKeyParent;

      // List book child.
      $bookChild = $this->bookOutlineStorage
        ->loadBookChildren($nidParentActive);
      $x = 0;
      foreach ($bookChild as $key => $child) {

        if ($key == $node->id()) {
          $contentNode['class'][$x] = 'active';
        }
        $childNode = $this->entityTypeManager
          ->getStorage('node')
          ->load($key);
        $contentNode['child_link'][$x] = $childNode->toLink($childNode->get('title')->value);
        $x = $x + 1;
      }

      $keys = array_keys($book);
      $nextSection = $keys[array_search($nidParentActive, $keys) + 1];

      // Next section.
      $nextNode = $this->entityTypeManager
          ->getStorage('node')
          ->load($nextSection);
      if (isset($nextNode)) {
        $contentNode['table_of_contents'] = $this->linkProvider->renderLink($nextNode->toUrl(), '<i class="fa fa-caret-left"></i> ',
            'LIST OF BOOKS');
        $contentNode['next_link'] = $nextNode->toLink($nextNode->get('title')->value);
      }
    }
    else {
      $contentNode['lbk_book_navigation_block'] = "LBK Book Navigation definition";
    }

    return $contentNode;

  }

  /**
   * Get Node Entity.
   *
   * @return \Drupal\node\Entity\Node
   *   Return Node Entity.
   */
  public function getNodeObject() {

    $node = '';
    // Get Node Entity object.
    if ($this->currentRouteMatch->getParameter('node')) {
      $node = $this->currentRouteMatch->getParameter('node');
    }
    else {
      $currentRoute = $this->currentRouteMatch;
      $sectionStorage = $currentRoute->getParameters()->get('section_storage');

      if ($sectionStorage instanceof OverridesSectionStorage) {
        $context = $sectionStorage->getContext('entity');
        if ($context instanceof EntityContext) {
          $contextData = $context->getContextData();
          if ($contextData instanceof EntityAdapter) {
            $node = $contextData->getEntity();
          }
        }
      }
    }

    return $node;
  }

}
