<?php

namespace Drupal\layout_builder_kit\Plugin\Block\LBKIconText;

use Drupal\Core\Config\ConfigManagerInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\file\Entity\File;
use Drupal\layout_builder_kit\Entity\LBKIcon;
use Drupal\layout_builder_kit\Plugin\Block\LBKBaseComponent;
use Symfony\Component\DependencyInjection\ContainerInterface;

use Drupal\block\Entity\Block;
use Drupal\Core\Entity\Plugin\DataType\EntityAdapter;
use Drupal\Core\Extension\Extension;
use Drupal\Core\Plugin\Context\EntityContext;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\layout_builder\Entity\LayoutBuilderEntityViewDisplay;
use Drupal\layout_builder\Plugin\SectionStorage\DefaultsSectionStorage;
use Drupal\layout_builder\Plugin\SectionStorage\OverridesSectionStorage;

/**
 * Provides a 'LBKIconText' block.
 *
 * @Block(
 *  id = "lbk_icon_text",
 *  admin_label = @Translation("Icon Text (LBK)"),
 * )
 */
class LBKIconText extends LBKBaseComponent implements ContainerFactoryPluginInterface {

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Drupal\Core\Config\ConfigManagerInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigManagerInterface
   */
  protected $configManager;

  /**
   * The database.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Constructs a new LBKIconText object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param string $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The EntityTypeManagerInterface service.
   * @param \Drupal\Core\Config\ConfigManagerInterface $config_manager
   *   The ConfigManagerInterface service.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    ConfigManagerInterface $config_manager,
    Connection $database
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->configManager = $config_manager;
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('config.manager'),
      $container->get('database')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'grid_fields' => [
        'grid_title' => 'Icon Text (LBK)',
        'grid_display_title' => TRUE,
        'grid_image' => [],
        'grid_position_media' => 'media_on_left',
        'grid_text' => [],
        'grid_link' => '',
        'grid_classes' => '',
      ],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   *
   * Creates a generic configuration form for all block types. Individual
   * block plugins can add elements to this form by overriding
   * BlockBase::blockForm(). Most block plugins should not override this
   * method unless they need to alter the generic form elements.
   *
   * @see \Drupal\Core\Block\BlockBase::blockForm()
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $definition = $this->getPluginDefinition();
    $form['provider'] = [
      '#type' => 'value',
      '#value' => $definition['provider'],
    ];

    // Add context mapping UI form elements.
    $contexts = $form_state->getTemporaryValue('gathered_contexts') ?: [];
    $form['context_mapping'] = $this->addContextAssignmentElement($this, $contexts);
    // Add plugin-specific settings for this block type.
    $form += $this->blockForm($form, $form_state);
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    // Get value from ImageComponentSettingsForm's configuration.
    $configFactory = $this->configManager->getConfigFactory();
    $locationFolderConfig = $configFactory->get('layout_builder_kit.image_component');
    $locationFolder = $locationFolderConfig->get('image_location');

    // Get the BlockID and query DB.
    $lbkIcon = NULL;
    $blockId = $this->configuration['grid_block_id'];

    $results = $this->database->select('lbk_icon_field_data', 'i')
      ->fields('i')
      ->condition('block_id', $blockId, '=')
      ->execute()
      ->fetchAll();

    if ($results) {
      $lbkIcon = LBKIcon::load($results[0]->id);

      $form['grid_title'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Title'),
        '#maxlength' => 255,
        '#default_value' => $lbkIcon->get('name')->value,
        '#required' => TRUE,
        '#weight' => '1',
        '#prefix' => '<div class="custom-icontext--component">',
      ];

      $form['grid_display_title'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Display title'),
        '#default_value' => $lbkIcon->get('display_title')->value,
        '#weight' => '2',
      ];

      $form['grid_image'] = [
        '#type' => 'managed_file',
        '#title' => $this->t('Image'),
        '#upload_location' => ($locationFolder != NULL) ? 'public://' . $locationFolder : 'public://layout_builder_kit',
        '#multiple' => FALSE,
        '#default_value' => [$lbkIcon->get('image_id')->value],
        '#upload_validators' => [
          'file_validate_extensions' => ['gif png jpg jpeg'],
        ],
        '#weight' => '3',
      ];

      $form['grid_position_media'] = [
        '#type' => 'select',
        '#title' => $this->t('Position media'),
        '#options' => [
          'media_on_left' => $this->t('Media on left'),
          'media_on_right' => $this->t('Media on right'),
        ],
        '#default_value' => $lbkIcon->get('position_media')->value,
        '#weight' => '4',
      ];

      $form['grid_link'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Link URL'),
        '#description' => $this->t('URL for title and icon.'),
        '#maxlength' => 64,
        '#default_value' => $lbkIcon->get('link_url')->value,
        '#weight' => '5',
      ];

      $form['grid_text'] = [
        '#type' => 'text_format',
        '#title' => $this->t('Text'),
        '#default_value' => $lbkIcon->get('text')->value,
        '#weight' => '6',
      ];

      $form['grid_classes'] = [
        '#type' => 'textfield',
        '#title' => $this->t('CSS class'),
        '#description' => $this->t('A list of CSS classes added to this component separated by spaces.'),
        '#maxlength' => 64,
        '#default_value' => $lbkIcon->get('css_class')->value,
        '#weight' => '7',
        '#suffix' => '</div>',
      ];
    }
    else {
      $form['grid_title'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Title'),
        '#maxlength' => 255,
        '#required' => TRUE,
        '#weight' => '1',
        '#prefix' => '<div class="custom-icontext--component">',
      ];

      $form['grid_display_title'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Display title'),
        '#weight' => '2',
      ];

      $form['grid_image'] = [
        '#type' => 'managed_file',
        '#title' => $this->t('Image'),
        '#upload_location' => ($locationFolder != NULL) ? 'public://' . $locationFolder : 'public://layout_builder_kit',
        '#multiple' => FALSE,
        '#upload_validators' => [
          'file_validate_extensions' => ['gif png jpg jpeg'],
        ],
        '#weight' => '3',
      ];

      $form['grid_position_media'] = [
        '#type' => 'select',
        '#title' => $this->t('Position media'),
        '#options' => [
          'media_on_left' => $this->t('Media on left'),
          'media_on_right' => $this->t('Media on right'),
        ],
        '#weight' => '4',
      ];

      $form['grid_link'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Link URL'),
        '#description' => $this->t('URL for title and icon.'),
        '#maxlength' => 64,
        '#weight' => '5',
      ];

      $form['grid_text'] = [
        '#type' => 'text_format',
        '#title' => $this->t('Text'),
        '#weight' => '6',
      ];

      $form['grid_classes'] = [
        '#type' => 'textfield',
        '#title' => $this->t('CSS class'),
        '#description' => $this->t('A list of CSS classes added to this component separated by spaces.'),
        '#maxlength' => 64,
        '#weight' => '7',
        '#suffix' => '</div>',
      ];
    }

    $form['#attached']['library'] = ['layout_builder_kit/image-grid-styling'];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    // Block Layout.
    $blockId =  $form_state->getStorage()['machine_name.initial_values']['id'];
    if ($blockId) {
      $this->configuration['grid_block_id'] = $blockId;
    } else {
      $blockId = $this->configuration['grid_block_id'];
    }

    // TODO:Query for BlockID.
    $results = $this->database->select('lbk_icon_field_data', 'i')
      ->fields('i')
      ->condition('block_id', $blockId, '=')
      ->execute()
      ->fetchAll();

    if ($results) {
      // TODO: UPDATE, if it exists.
      foreach ($results as $row) {
        $iconId = $row->id;
      }

      $lbkIcon = LBKIcon::load($iconId);
      $lbkIcon->set('name', $form_state->getValue('grid_title'));
      $lbkIcon->set('display_title', $form_state->getValue('grid_display_title'));
      $lbkIcon->set('image_id', $form_state->getValue('grid_image')[0]);
      $lbkIcon->set('position_media', $form_state->getValue('grid_position_media'));
      $lbkIcon->set('link_url', $form_state->getValue('grid_link'));
      $lbkIcon->set('text', [
        'value' => $form_state->getValue('grid_text')['value'],
        'format' => $form_state->getValue('grid_text')['format'],
      ]);
      $lbkIcon->set('css_class', $form_state->getValue('grid_classes'));

      $lbkIcon->save();
    }
    else {
      // TODO: CREATE, if it does not exist.
      $lbkIcon = LBKIcon::create([
        'type' => 'icon',
        'block_id' => $blockId,
        'name' => $form_state->getValue('grid_title'),
        'display_title' => $form_state->getValue('grid_display_title'),
        'image_id' => $form_state->getValue('grid_image')[0],
        'position_media' => $form_state->getValue('grid_position_media'),
        'link_url' => $form_state->getValue('grid_link'),
        'text' => [
          'value' => $form_state->getValue('grid_text')['value'],
          'format' => $form_state->getValue('grid_text')['format'],
        ],
        'css_class' => $form_state->getValue('grid_classes')
      ]);

      $lbkIcon->save();


      if($blockId == NULL) {
        $this->configuration['grid_block_id'] = $lbkIcon->id();
        $lbkIcon->set('block_id', $lbkIcon->id());
        $lbkIcon->save();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];
    // Add template.
    $build['#theme'] = 'LBKIconText';
    // Attach library.
    $build['#attached']['library'] = ['layout_builder_kit/image-grid-styling'];

    // TODO: Get entity.
    $blockId =  $this->configuration['grid_block_id'];
    // TODO:Query for BlockID.
    $results = $this->database->select('lbk_icon_field_data', 'i')
      ->fields('i')
      ->condition('block_id', $blockId, '=')
      ->execute()
      ->fetchAll();

    if ($results) {
      foreach ($results as $row) {
        $iconId = $row->id;
      }

      $lbkIcon = LBKIcon::load($iconId);

      if (!empty($lbkIcon->get('image_id')->value)) {
        $gridImage = File::load($lbkIcon->get('image_id')->value);
        if ($gridImage != NULL) {
          $gridImage->setPermanent();
          $gridImage->save();

          $build['#image'] = [
            '#theme' => 'image',
            '#uri' => $gridImage->getFileUri(),
          ];
        }
      }

      $build['#title'] = $lbkIcon->get('name')->value;
      $build['#display_title'] = $lbkIcon->get('display_title')->value;;
      $build['#position_media'] = $lbkIcon->get('position_media')->value;
      $build['#text']['#markup'] = $lbkIcon->get('text')->value;
      $build['#link'] = $lbkIcon->get('link_url')->value;
      $build['#classes'] = $lbkIcon->get('css_class')->value;
    }

    return $build;
  }

}
