<?php

namespace Drupal\layout_builder_kit\Plugin\Block\LBKImage;

use Drupal\Core\Config\ConfigManagerInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\file\Entity\File;
use Drupal\layout_builder_kit\Entity\LBKImageEntity;
use Drupal\layout_builder_kit\Plugin\Block\LBKBaseComponent;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'LBKImage' block.
 *
 * @Block(
 *  id = "lbk_image",
 *  admin_label = @Translation("Image (LBK)"),
 * )
 */
class LBKImage extends LBKBaseComponent implements ContainerFactoryPluginInterface {

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Drupal\Core\Config\ConfigManagerInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigManagerInterface
   */
  protected $configManager;

  /**
   * The database.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Constructs a new GridComponent object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param string $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The EntityTypeManagerInterface service.
   * @param \Drupal\Core\Config\ConfigManagerInterface $config_manager
   *   The ConfigManagerInterface service.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    ConfigManagerInterface $config_manager,
    Connection $database
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->configManager = $config_manager;
    $this->database = $database;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('config.manager'),
      $container->get('database')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'image_component_fields' => [
        'title' => 'Image (LBK)',
        'display_title' => TRUE,
        'image' => [],
        'image_style' => 'none',
        'position' => 'left',
        'text' => [],
        'overlay_text' => 'none',
        'title_position' => 'title_on_top',
        'classes' => '',
      ],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   *
   * Creates a generic configuration form for all block types. Individual
   * block plugins can add elements to this form by overriding
   * BlockBase::blockForm(). Most block plugins should not override this
   * method unless they need to alter the generic form elements.
   *
   * @see \Drupal\Core\Block\BlockBase::blockForm()
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $definition = $this->getPluginDefinition();
    $form['provider'] = [
      '#type' => 'value',
      '#value' => $definition['provider'],
    ];

    // Add context mapping UI form elements.
    $contexts = $form_state->getTemporaryValue('gathered_contexts') ?: [];
    $form['context_mapping'] = $this->addContextAssignmentElement($this, $contexts);
    // Add plugin-specific settings for this block type.
    $form += $this->blockForm($form, $form_state);
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    // Get value from ImageComponentSettingsForm's configuration.
    $configFactory = $this->configManager->getConfigFactory();
    $locationFolderConfig = $configFactory->get('layout_builder_kit.image_component');
    $locationFolder = $locationFolderConfig->get('image_location');

    // Get the BlockID and query DB.
    $lbkIcon = NULL;
    $blockId = $this->configuration['grid_block_id'];

    $results = $this->database->select('lbk_image_entity_field_data', 'i')
      ->fields('i')
      ->condition('block_id', $blockId, '=')
      ->execute()
      ->fetchAll();

    if ($results) {
      $lbkImage = LBKImageEntity::load($results[0]->id);

      $form['title'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Title'),
        '#maxlength' => 255,
        '#default_value' => $lbkImage->get('name')->value,
        '#required' => TRUE,
        '#weight' => '1',
        '#prefix' => '<div class="custom-image--component">',
      ];

      $form['display_title'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Display title'),
        '#default_value' => $lbkImage->get('display_title')->value,
        '#weight' => '2',
      ];

      if ($lbkImage->get('display_title')->value) {
        $form['title_position'] = [
          '#type' => 'select',
          '#title' => $this->t('Title alignment'),
          '#options' => [
            'title_on_top' => $this->t('Top'),
            'title_on_bottom' => $this->t('Bottom'),
          ],
          '#default_value' => $lbkImage->get('title_alignment')->value,
          '#weight' => '3',
        ];
      }

      $form['image'] = [
        '#type' => 'managed_file',
        '#title' => $this->t('Image'),
        '#upload_location' => ($locationFolder != NULL) ? 'public://' . $locationFolder : 'public://layout_builder_kit',
        '#multiple' => FALSE,
        '#default_value' => [$lbkImage->get('image_id')->value],
        '#required' => TRUE,
        '#upload_validators' => [
          'file_validate_extensions' => ['gif png jpg jpeg'],
        ],
        '#weight' => '4',
      ];

      $form['image_style'] = [
        '#type' => 'select',
        '#title' => $this->t('Image style'),
        '#options' => [
          'none' => $this->t('None'),
          'thumbnail' => $this->t('Thumbnail'),
          'medium' => $this->t('Medium'),
          'large' => $this->t('Large'),
        ],
        '#default_value' => $lbkImage->get('image_style')->value,
        '#weight' => '5',
      ];

      $form['position'] = [
        '#type' => 'select',
        '#title' => $this->t('Alignment'),
        '#options' => [
          'left' => $this->t('Left'),
          'center' => $this->t('Center'),
          'right' => $this->t('Right'),
        ],
        '#default_value' => $lbkImage->get('alignment')->value,
        '#weight' => '6',
      ];

      $form['text'] = [
        '#type' => 'text_format',
        '#title' => $this->t('Overlay text'),
        '#default_value' => $lbkImage->get('overlay_text')->value,
        //'#format' => $this->configuration['image_component_fields']['text']['format'],
        '#weight' => '7',
      ];

      $form['overlay_text'] = [
        '#type' => 'select',
        '#title' => $this->t('Overlay text position'),
        '#options' => [
          'none' => $this->t('None'),
          'bottom' => $this->t('Bottom'),
        ],
        '#default_value' => $lbkImage->get('overlay_text_position')->value,
        '#weight' => '8',
      ];

      $form['classes'] = [
        '#type' => 'textfield',
        '#title' => $this->t('CSS class'),
        '#description' => $this->t('A list of CSS classes added to this component separated by spaces.'),
        '#maxlength' => 64,
        '#default_value' => $lbkImage->get('css_class')->value,
        '#weight' => '9',
        '#suffix' => '</div>',
      ];
    }
    else {
      $form['title'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Title'),
        '#maxlength' => 255,
        '#required' => TRUE,
        '#weight' => '1',
        '#prefix' => '<div class="custom-image--component">',
      ];

      $form['display_title'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Display title'),
        '#weight' => '2',
      ];

      $form['image'] = [
        '#type' => 'managed_file',
        '#title' => $this->t('Image'),
        '#upload_location' => ($locationFolder != NULL) ? 'public://' . $locationFolder : 'public://layout_builder_kit',
        '#multiple' => FALSE,
        '#required' => TRUE,
        '#upload_validators' => [
          'file_validate_extensions' => ['gif png jpg jpeg'],
        ],
        '#weight' => '4',
      ];

      $form['image_style'] = [
        '#type' => 'select',
        '#title' => $this->t('Image style'),
        '#options' => [
          'none' => $this->t('None'),
          'thumbnail' => $this->t('Thumbnail'),
          'medium' => $this->t('Medium'),
          'large' => $this->t('Large'),
        ],
        '#weight' => '5',
      ];

      $form['position'] = [
        '#type' => 'select',
        '#title' => $this->t('Alignment'),
        '#options' => [
          'left' => $this->t('Left'),
          'center' => $this->t('Center'),
          'right' => $this->t('Right'),
        ],
        '#weight' => '6',
      ];

      $form['text'] = [
        '#type' => 'text_format',
        '#title' => $this->t('Overlay text'),
        '#weight' => '7',
      ];

      $form['overlay_text'] = [
        '#type' => 'select',
        '#title' => $this->t('Overlay text position'),
        '#options' => [
          'none' => $this->t('None'),
          'bottom' => $this->t('Bottom'),
        ],
        '#weight' => '8',
      ];

      $form['classes'] = [
        '#type' => 'textfield',
        '#title' => $this->t('CSS class'),
        '#description' => $this->t('A list of CSS classes added to this component separated by spaces.'),
        '#maxlength' => 64,
        '#weight' => '9',
        '#suffix' => '</div>',
      ];
    }

    $form['#attached']['library'] = ['layout_builder_kit/image-styling'];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {

    if (!$form_state->getValue('display_title')) {
      $this->configuration['image_component_fields']['display_title'] = $form_state->getValue('display_title');
      $this->configuration['image_component_fields']['title_position'] = 'title_on_top';

      $displayTitle = $form_state->getValue('display_title');
      $titlePosition = 'title_on_top';
    }
    else {
      $this->configuration['image_component_fields']['display_title'] = $form_state->getValue('display_title');
      $this->configuration['image_component_fields']['title_position'] = $form_state->getValue('title_position');

      $displayTitle = $form_state->getValue('display_title');
      $titlePosition = $form_state->getValue('title_position');
    }

    $blockId =  $form_state->getStorage()['machine_name.initial_values']['id'];
    if ($blockId) {
      $this->configuration['grid_block_id'] = $blockId;
    } else {
      $blockId = $this->configuration['grid_block_id'];
    }

    // TODO:Query for BlockID.
    $results = $this->database->select('lbk_image_entity_field_data', 'i')
      ->fields('i')
      ->condition('block_id', $blockId, '=')
      ->execute()
      ->fetchAll();


    if ($results) {
      // TODO: UPDATE, if it exists.
      $lbkIcon = LBKImageEntity::load($results[0]->id);
      $lbkIcon->set('name', $form_state->getValue('title'));
      $lbkIcon->set('image_id', $form_state->getValue('image')[0]);
      $lbkIcon->set('title_alignment', $titlePosition);
      $lbkIcon->set('image_style', $form_state->getValue('image_style'));
      $lbkIcon->set('overlay_text', [
        'value' => $form_state->getValue('text')['value'],
        'format' => $form_state->getValue('text')['format'],
      ]);
      $lbkIcon->set('overlay_text_position', $form_state->getValue('overlay_text'));
      $lbkIcon->set('css_class', $form_state->getValue('classes'));
      $lbkIcon->set('display_title', $displayTitle);

      $lbkIcon->save();
    }
    else {
      // TODO: CREATE, if it does not exist.
      $lbkIcon =  LBKImageEntity::create([
        'type' => 'image',
        'block_id' => $blockId,
        'name' => $form_state->getValue('title'),
        'image_id' => $form_state->getValue('image')[0],
        'title_alignment' => $titlePosition,
        'image_style' => $form_state->getValue('image_style'),
        'alignment' => $form_state->getValue('position'),
        'overlay_text' => [
          'value' => $form_state->getValue('text')['value'],
          'format' => $form_state->getValue('text')['format'],
        ],
        'overlay_text_position' => $form_state->getValue('overlay_text'),
        'css_class' => $form_state->getValue('classes'),
        'display_title' => $displayTitle
      ]);

      $lbkIcon->save();

      if($blockId == NULL) {
        $this->configuration['grid_block_id'] = $lbkIcon->id();
        $lbkIcon->set('block_id', $lbkIcon->id());
        $lbkIcon->save();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];
    // Add template.
    $build['#theme'] = 'LBKImage';
    // Attach library.
    $build['#attached']['library'] = ['layout_builder_kit/image-styling'];

    // TODO: Get entity.
    $blockId =  $this->configuration['grid_block_id'];
    // TODO:Query for BlockID.
    $results = $this->database->select('lbk_image_entity_field_data', 'i')
      ->fields('i')
      ->condition('block_id', $blockId, '=')
      ->execute()
      ->fetchAll();

    if ($results) {
      $lbkIcon = LBKImageEntity::load($results[0]->id);

      if (!empty($lbkIcon->get('image_id')->value)) {
        $gridImage = File::load($lbkIcon->get('image_id')->value);
        if ($gridImage != NULL) {
          $gridImage->setPermanent();
          $gridImage->save();

          $build['#image'] = [
            '#theme' => 'image',
            '#uri' => $gridImage->getFileUri(),
          ];
        }
      }

      $build['#title'] = $lbkIcon->get('name')->value;
      $build['#display_title'] = $lbkIcon->get('display_title')->value;
      $build['#position'] = $lbkIcon->get('alignment')->value;
      $build['#text']['#markup'] = $lbkIcon->get('overlay_text')->value;
      $build['#overlay_text'] = $lbkIcon->get('overlay_text_position')->value;
      $build['#title_position'] = $lbkIcon->get('title_alignment')->value;
      $build['#classes'] = $lbkIcon->get('css_class')->value;
    }

    return $build;
  }

}
