<?php

namespace Drupal\layout_builder_kit\Plugin\Block\LBKTab;

use Drupal\block\Entity\Block;
use Drupal\block_content\Entity\BlockContent;
use Drupal\Core\Config\ConfigManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\Plugin\DataType\ConfigEntityAdapter;
use Drupal\Core\Entity\Plugin\DataType\EntityAdapter;
use Drupal\Core\Extension\Extension;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\Context\EntityContext;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\layout_builder\Entity\LayoutBuilderEntityViewDisplay;
use Drupal\layout_builder\Plugin\SectionStorage\DefaultsSectionStorage;
use Drupal\layout_builder\Plugin\SectionStorage\OverridesSectionStorage;
use Drupal\layout_builder_kit\Plugin\Block\LBKBaseComponent;
use Drupal\node\Entity\Node;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'LBKTab' block.
 *
 * @Block(
 *  id = "lbk_tab",
 *  admin_label = @Translation("Tab (LBK)"),
 * )
 */
class LBKTab extends LBKBaseComponent implements ContainerFactoryPluginInterface {

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Drupal\Core\Config\ConfigManagerInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigManagerInterface
   */
  protected $configManager;

  /**
   * Drupal\Core\Routing\CurrentRouteMatch class.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  /**
   * Constructs a new GridComponent object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param string $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The EntityTypeManagerInterface service.
   * @param \Drupal\Core\Config\ConfigManagerInterface $config_manager
   *   The ConfigManagerInterface service.
   * @param \Drupal\Core\Routing\CurrentRouteMatch $currentRouteMatch
   *   The CurrentRouteMatch service.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    ConfigManagerInterface $config_manager,
    CurrentRouteMatch $currentRouteMatch
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->configManager = $config_manager;
    $this->currentRouteMatch = $currentRouteMatch;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('config.manager'),
      $container->get('current_route_match')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'lbk_tab' => [
        'tab_content' => [],
      ],
      'tabs_default_text' => $this->t('No tab content.'),
      'title' => 'Tab (LBK)',
      'display_title' => TRUE,
      'classes' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   *
   * Creates a generic configuration form for all block types. Individual
   * block plugins can add elements to this form by overriding
   * BlockBase::blockForm(). Most block plugins should not override this
   * method unless they need to alter the generic form elements.
   *
   * @see \Drupal\Core\Block\BlockBase::blockForm()
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $definition = $this->getPluginDefinition();
    $form['provider'] = [
      '#type' => 'value',
      '#value' => $definition['provider'],
    ];

    // Add context mapping UI form elements.
    $contexts = $form_state->getTemporaryValue('gathered_contexts') ?: [];
    $form['context_mapping'] = $this->addContextAssignmentElement($this, $contexts);
    // Add plugin-specific settings for this block type.
    $form += $this->blockForm($form, $form_state);
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $config = $this->getConfiguration();

    $form['title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Title'),
      '#maxlength' => 255,
      '#default_value' => $this->configuration['title'],
      '#required' => TRUE,
      '#weight' => '-2',
      '#prefix' => '<div class="custom-tab--component">',
    ];

    $form['display_title'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display title'),
      '#default_value' => $this->configuration['display_title'],
      '#weight' => '-1',
    ];

    $form['#tree'] = TRUE;
    $form['items_fieldset'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Tabs'),
      '#prefix' => '<div id="items-fieldset-wrapper">',
      '#suffix' => '</div>',
      '#weight' => '0',
    ];

    if (!$form_state->has('items')) {
      $form_state->set('items', $config['lbk_tab']['tab_content']);
    }

    $tabs = $form_state->get('items');
    $numTabs = count($tabs);
    for ($i = 0; $i < $numTabs; $i++) {

      if ($tabs[$i]['type'] === 'custom_text') {
        // Tab label.
        $form['items_fieldset']['items'][$i]['name_tab'] = [
          '#type' => 'textfield',
          '#title' => $this->t('Tab @index label', ['@index' => $i + 1]),
          '#default_value' => $tabs[$i]['name_tab'],
          '#maxlength' => 64,
          '#size' => 64,
          '#weight' => '0',
        ];

        // Content Tab.
        $form['items_fieldset']['items'][$i]['text_format'] = [
          '#type' => 'text_format',
          '#title' => $this->t('Rich text'),
          '#default_value' => $tabs[$i]['text_format']['value'],
          '#format' => $tabs[$i]['text_format']['format'],
          '#weight' => '1',
        ];

      }
      else {
        if ($tabs[$i]['type'] === 'block') {
          // Tab label.
          $form['items_fieldset']['items'][$i]['name_tab'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Tab @index label', ['@index' => $i + 1]),
            '#default_value' => $tabs[$i]['name_tab'],
            '#maxlength' => 64,
            '#size' => 64,
            '#weight' => '0',
          ];

          // Block Tab.
          $form['items_fieldset']['items'][$i]['name'] = [
            '#type' => 'select',
            '#title' => $this->t('Content Block'),
            '#options' => $this->getBlocks(),
            '#default_value' => $tabs[$i]['name'],
            '#weight' => '1',
          ];

        }
      }
    }

    if ($numTabs >= 1) {
      $form['items_fieldset']['items']['remove_item'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove last tab'),
        '#submit' => [[$this, 'removeLastBlock']],
        '#ajax' => [
          'callback' => [$this, 'removeLastBlockCallback'],
          'wrapper' => 'items-fieldset-wrapper',
        ],
        '#weight' => '2',
      ];
    }

    $form['items_fieldset']['actions'] = [
      '#type' => 'actions',
    ];

    $form['add_custom_text'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add custom text'),
      '#submit' => [[$this, 'addCustomText']],
      '#ajax' => [
        'callback' => [$this, 'addCustomTextCallback'],
        'wrapper' => 'items-fieldset-wrapper',
      ],
      '#weight' => '1',
      '#prefix' => '<div class="lbk-tab-buttons">',
    ];

    $form['add_block_tab'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add block'),
      '#submit' => [[$this, 'addBlock']],
      '#ajax' => [
        'callback' => [$this, 'addBlockCallback'],
        'wrapper' => 'items-fieldset-wrapper',
      ],
      '#weight' => '1',
      '#suffix' => '</div>',
    ];

    $form['tabs_default_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('"No tabs" text.'),
      '#default_value' => $this->configuration['tabs_default_text'],
      '#maxlength' => 64,
      '#size' => 64,
      '#weight' => '2',
    ];

    $form['classes'] = [
      '#type' => 'textfield',
      '#title' => $this->t('CSS class'),
      '#description' => $this->t('A list of CSS classes added to this component separated by spaces.'),
      '#maxlength' => 64,
      '#default_value' => $this->configuration['classes'],
      '#weight' => '3',
    ];

    $form['#attached']['library'] = ['layout_builder_kit/tab-styling'];

    return $form;
  }

  /**
   * Add a Block as Custom.
   *
   * @param array $form
   *   The array form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The FormStateInterface.
   */
  public function addCustomText(array &$form, FormStateInterface $form_state) {
    $items = [];
    if ($form_state->has('items')) {
      $items = $form_state->get('items');
      $nextItem = count($items);
      $items[$nextItem]['type'] = 'custom_text';
    }
    else {
      $nextItem = count($items);
      $items[$nextItem]['type'] = 'custom_text';
    }

    $form_state->set('items', $items);
    $form_state->setRebuild();
  }

  /**
   * Add Custom Text Callback.
   *
   * @param array $form
   *   The array form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The FormStateInterface.
   *
   * @return mixed
   *   Return the form.
   */
  public function addCustomTextCallback(array &$form, FormStateInterface $form_state) {
    return $form['settings']['items_fieldset'];
  }

  /**
   * Add any Block.
   *
   * @param array $form
   *   The array form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The FormStateInterface.
   */
  public function addBlock(array &$form, FormStateInterface $form_state) {
    $items = [];
    if ($form_state->has('items')) {
      $items = $form_state->get('items');
      $nextItem = count($items);
      $items[$nextItem]['type'] = 'block';
    }
    else {
      $nextItem = count($items);
      $items[$nextItem]['type'] = 'block';
    }

    $form_state->set('items', $items);
    $form_state->setRebuild();
  }

  /**
   * Add Block Callback.
   *
   * @param array $form
   *   The array form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The FormStateInterface.
   *
   * @return mixed
   *   Return the form.
   */
  public function addBlockCallback(array &$form, FormStateInterface $form_state) {
    return $form['settings']['items_fieldset'];
  }

  /**
   * Remove last Block.
   *
   * @param array $form
   *   The array form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The FormStateInterface.
   */
  public function removeLastBlock(array &$form, FormStateInterface $form_state) {
    $items = [];
    if ($form_state->has('items')) {
      $items = $form_state->get('items');
      array_pop($items);
    }

    $form_state->set('items', $items);
    $form_state->setRebuild();
  }

  /**
   * Remove Last Block Callback.
   *
   * @param array $form
   *   The array form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The FormStateInterface.
   *
   * @return mixed
   *   Return the form.
   */
  public function removeLastBlockCallback(array &$form, FormStateInterface $form_state) {
    return $form['settings']['items_fieldset'];
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['title'] = $form_state->getValue('title');
    $this->configuration['display_title'] = $form_state->getValue('display_title');
    $this->configuration['tabs_default_text'] = $form_state->getValue('tabs_default_text');
    $this->configuration['classes'] = $form_state->getValue('classes');

    // Set to empty before save all again.
    $this->configuration['lbk_tab']['tab_content'] = [];

    foreach ($form_state->getValues() as $key => $value) {
      if ($key === 'items_fieldset') {
        if (isset($value['items'])) {
          $items = $value['items'];
          $tabs = $form_state->get('items');
          if (is_array($tabs)) {
            for ($i = 0; $i < count($tabs); $i++) {
              if ($tabs[$i]['type'] === 'custom_text') {
                $this->configuration['lbk_tab']['tab_content'][$i]['type'] = 'custom_text';
                $this->configuration['lbk_tab']['tab_content'][$i]['name_tab'] = $items[$i]['name_tab'];
                $this->configuration['lbk_tab']['tab_content'][$i]['text_format']['value'] = $items[$i]['text_format']['value'];
                $this->configuration['lbk_tab']['tab_content'][$i]['text_format']['format'] = $items[$i]['text_format']['format'];
              }
              else {
                if ($tabs[$i]['type'] === 'block') {
                  $this->configuration['lbk_tab']['tab_content'][$i]['type'] = 'block';
                  $this->configuration['lbk_tab']['tab_content'][$i]['name_tab'] = $items[$i]['name_tab'];
                  $this->configuration['lbk_tab']['tab_content'][$i]['name'] = $items[$i]['name'];
                }
              }
            }
          }
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $tabs = [];
    $node = NULL;
    $renderedBlock = NULL;
    if (isset($this->configuration['lbk_tab']['tab_content'])) {
      for ($i = 0; $i < count($this->configuration['lbk_tab']['tab_content']); $i++) {
        if ($this->configuration['lbk_tab']['tab_content'][$i]['type'] === 'block') {
          $tabs[$i]['type'] = $this->configuration['lbk_tab']['tab_content'][$i]['type'];
          $tabs[$i]['name_tab'] = $this->configuration['lbk_tab']['tab_content'][$i]['name_tab'];

          // Render Block.
          if ((int) $this->configuration['lbk_tab']['tab_content'][$i]['name'] > 0) {
            // Render Content Block.
            $bid = $this->configuration['lbk_tab']['tab_content'][$i]['name'];
            $block = BlockContent::load($bid);
            if ($block instanceof BlockContent) {
              $renderedBlock = \Drupal::entityTypeManager()->getViewBuilder('block_content')->view($block);
            }
            else {
              if ($this->configuration['lbk_tab']['tab_content'][$i]['name'] == 'none') {
                $renderedBlock = $this->t("There is no any content block yet.");
              }
              else {
                $renderedBlock = $this->t("Sorry this Block was deleted.");
              }
            }
          }
          else {
            // Render fields.
            $field = $this->getField($this->configuration['lbk_tab']['tab_content'][$i]['name']);

            if (isset($field) && $field !== "") {
              // View builder.
              $entityType = 'node';
              // Fields to render.
              $fields = [
                $field,
              ];

              // Get the Node entity object.
              if (\Drupal::routeMatch()->getParameter('node')) {
                $node = \Drupal::routeMatch()->getParameter('node');
              }
              else {
                $currentRoute = $this->currentRouteMatch;
                $sectionStorage = $currentRoute->getParameters()->get('section_storage');
                if ($sectionStorage instanceof OverridesSectionStorage) {
                  $context = $sectionStorage->getContext('entity');
                  if ($context instanceof EntityContext) {
                    $contextData = $context->getContextData();
                    if ($contextData instanceof EntityAdapter) {
                      $node = $contextData->getEntity();
                    }
                  }
                }
              }

              // Get the View for this Node entity object.
              $viewBuilder = $this->entityTypeManager->getViewBuilder($entityType);

              // Build the Node array to render.
              foreach ($fields as $field_name) {
                if ($node instanceof Node) {
                  if ($node->hasField($field_name) && $node->access('view')) {
                    $value = $node->get($field_name);
                    $renderedBlock = $viewBuilder->viewField($value, ["label" => "hidden"]);
                    $renderedBlock['#cache']['tags'] = $node->getCacheTags();
                  }
                  else {
                    $renderedBlock = $this->t('This field is not available for this content.');
                  }
                }
                else {
                  $renderedBlock = $this->t('Render this inside a Node page that has this field.');
                }
              }
            }
            else {
              // Render any content block.
              $blockEntity = Block::load($this->configuration['lbk_tab']['tab_content'][$i]['name']);
              $renderedBlock = \Drupal::entityTypeManager()
                ->getViewBuilder('block')
                ->view($blockEntity);
            }
          }
          $tabs[$i]['name'] = $renderedBlock;
        }
        else {
          $tabs[$i] = $this->configuration['lbk_tab']['tab_content'][$i];
        }
      }
    }

    $build = [];
    $build['#theme'] = 'LBKTab';
    $build['#attached']['library'] = ['layout_builder_kit/tab-styling'];
    $build['#title'] = $this->configuration['title'];
    $build['#display_title'] = $this->configuration['display_title'];
    $build['#tabs_default_text']['#markup'] = $this->configuration['tabs_default_text'];
    $build['#classes'] = $this->configuration['classes'];
    $build['#tabs'] = $tabs;
    return $build;
  }

  /**
   * Get all Content Blocks.
   *
   * @return array
   *   Return array with Content Blocks.
   */
  private function getContentBlocks() {
    $blocks = BlockContent::loadMultiple();

    $options = [];
    $labelOption = $this->t('Custom');

    if ($labelOption instanceof TranslatableMarkup) {
      foreach ($blocks as $key => $value) {
        if ($value instanceof BlockContent) {
          $options[$labelOption->render()][$value->id()] = $value->label();
        }
      }
    }

    return $options;
  }

  /**
   * Get all Content Plugin Blocks.
   *
   * @return array
   *   Return array with Content Plugin Blocks.
   */
  private function getContentPluginBlocks() {
    // Get in Drop-down all Content Plugin Blocks.
    $blocksList = [];
    $labelOption = $this->t('Modules');

    // Load all Plugin Block entities.
    $blocks = Block::loadMultiple();

    $moduleHandler = \Drupal::moduleHandler();
    $modules = $moduleHandler->getModuleList();

    if ($labelOption instanceof TranslatableMarkup) {
      foreach ($blocks as $block) {
        if ($block instanceof Block) {
          $dependenciesBlock = $block->getDependencies()['module'];
          $system = FALSE;
          foreach ($dependenciesBlock as $dependencyBlock) {
            $module = $modules[$dependencyBlock];
            if ($module instanceof Extension) {
              if ($block->getPluginId() !== 'lbk_tab') {
                $pathModuleArray = explode('/', $module->getPathname());
                if (in_array('contrib', $pathModuleArray) || in_array('custom', $pathModuleArray)) {
                  $system = TRUE;
                }
              }
            }
          }
          if ($system) {
            $blocksList[$labelOption->render()][$block->id()] = $block->label();
          }
        }
      }
    }

    return $blocksList;
  }

  /**
   * Get all System Plugin Blocks.
   *
   * @return array
   *   Return array with System Plugin Blocks.
   */
  private function getSystemPluginBlocks() {
    // Get in Drop-down all Content Plugin Blocks.
    $blocksList = [];
    $labelOption = $this->t('System');

    // Load all Plugin Block entities.
    $blocks = Block::loadMultiple();

    $moduleHandler = \Drupal::moduleHandler();
    $modules = $moduleHandler->getModuleList();
    unset($modules['layout_builder_kit']);

    if ($labelOption instanceof TranslatableMarkup) {
      foreach ($blocks as $block) {
        if ($block instanceof Block) {
          $dependenciesBlock = $block->getDependencies()['module'];
          $system = FALSE;
          foreach ($dependenciesBlock as $dependencyBlock) {
            $module = $modules[$dependencyBlock];
            if ($module instanceof Extension) {
              $pathModuleArray = explode('/', $module->getPathname());
              if (in_array('core', $pathModuleArray)) {
                $system = TRUE;
              }
            }
          }
          if ($system) {
            $blocksList[$labelOption->render()][$block->id()] = ucfirst($block->getTheme()) . ':' . $block->label();
          }
        }
      }
    }

    return $blocksList;
  }

  /**
   * Get All Blocks together.
   *
   * @return array
   *   Return all blocks.
   */
  private function getBlocks() {
    $contentAndContentPluginBlocks = array_merge($this->getContentBlocks(), $this->getContentPluginBlocks());
    $allBlocks = array_merge($contentAndContentPluginBlocks, $this->getSystemPluginBlocks());

    $allFields = array_merge($allBlocks, $this->getNodeFields());

    return $allFields;
  }

  /**
   * Get Node's fields.
   *
   * @return array
   *   Return Node's fields in an array.
   */
  private function getNodeFields() {
    $allFields = [];
    // Get the Current Route Match.
    $currentRoute = $this->currentRouteMatch;
    if ($currentRoute instanceof CurrentRouteMatch) {
      $sectionStorage = $currentRoute->getParameters()->get('section_storage');
      // Content Type page.
      if ($sectionStorage instanceof DefaultsSectionStorage) {
        $availableContexts = $sectionStorage->getContexts();
        $contextDisplay = $availableContexts['display'];
        if ($contextDisplay instanceof EntityContext) {
          $contextData = $contextDisplay->getContextData();
          if ($contextData instanceof ConfigEntityAdapter) {
            $contentInfo = $contextData->getEntity();
            if ($contentInfo instanceof LayoutBuilderEntityViewDisplay) {
              // Bundle 'page'.
              $bundle = $contentInfo->getTargetBundle();
              // Content Type 'node'.
              $entityTypeId = $contentInfo->getTargetEntityTypeId();
              foreach (\Drupal::entityManager()->getFieldDefinitions($entityTypeId, $bundle) as $fieldName => $fieldDefinition) {
                if (!empty($fieldDefinition->getTargetBundle()) && $fieldName !== 'layout_builder__layout') {
                  $allFields[$bundle][$bundle . '_' . $fieldName] = $bundle . ':' . $fieldDefinition->getLabel();
                }
              }
            }
          }
        }
      }
      else {
        // Bundle page.
        if ($sectionStorage instanceof OverridesSectionStorage) {
          $context = $sectionStorage->getContext('entity');
          if ($context instanceof EntityContext) {
            $contextData = $context->getContextData();
            if ($contextData instanceof EntityAdapter) {
              $node = $contextData->getEntity();
              if ($node instanceof Node) {
                $bundle = $node->bundle();
                foreach (\Drupal::entityManager()->getFieldDefinitions('node', $bundle) as $fieldName => $fieldDefinition) {
                  if (!empty($fieldDefinition->getTargetBundle()) && $fieldName !== 'layout_builder__layout') {
                    $allFields[$bundle][$bundle . '_' . $fieldName] = $bundle . ':' . $fieldDefinition->getLabel();
                  }
                }
              }
            }
          }
        }
        else {
          // Block Layout.
          // Get fields for all bundles.
          $entityType = "node";
          $contentTypes = \Drupal::entityManager()->getBundleInfo($entityType);

          foreach ($contentTypes as $key => $value) {
            $bundle = $key;
            foreach (\Drupal::entityManager()->getFieldDefinitions($entityType, $bundle) as $fieldName => $fieldDefinition) {
              if (!empty($fieldDefinition->getTargetBundle())) {
                $allFields[$value['label']][$bundle . '_' . $fieldName] = $value['label'] . ':' . $fieldDefinition->getLabel();
              }
            }
          }
        }
      }
    }

    return $allFields;
  }

  /**
   * Get machine name of field.
   *
   * @param string $fieldName
   *   The fieldName concatenated with bundle.
   *
   * @return mixed|string
   *   Return the machine name.
   */
  private function getField($fieldName) {
    $machineName = '';
    // Get fields for all bundles.
    $entityType = "node";
    $contentTypes = \Drupal::entityManager()->getBundleInfo($entityType);

    foreach ($contentTypes as $key => $value) {
      if (strpos($fieldName, $key . '_') !== FALSE) {
        $machineName = str_replace($key . '_', '', $fieldName);
      }
    }

    return $machineName;
  }

}
