<?php

namespace Drupal\layout_builder_kit\Plugin\Block\LBKVideo;

use Drupal\Core\Config\ConfigManagerInterface;
use Drupal\Core\Entity\EntityFieldManager;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\Plugin\DataType\EntityAdapter;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\Context\EntityContext;
use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\FieldConfigInterface;
use Drupal\file\Entity\File;
use Drupal\layout_builder\Plugin\SectionStorage\OverridesSectionStorage;
use Drupal\layout_builder_kit\Plugin\Block\LBKBaseComponent;
use Drupal\node\Entity\Node;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'LBKVideo' block.
 *
 * @Block(
 *  id = "lbk_video",
 *  admin_label = @Translation("Video (LBK)"),
 * )
 */
class LBKVideo extends LBKBaseComponent implements ContainerFactoryPluginInterface {

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Drupal\Core\Config\ConfigManagerInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigManagerInterface
   */
  protected $configManager;

  /**
   * Drupal\Core\Routing\CurrentRouteMatch class.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  /**
   * Drupal\Core\Entity\EntityFieldManager definition.
   *
   * @var \Drupal\Core\Entity\EntityFieldManager
   */
  protected $entityFieldManager;

  /**
   * Constructs a new GridComponent object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $pluginId
   *   The plugin_id for the plugin instance.
   * @param string $pluginDefinition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The EntityTypeManagerInterface service.
   * @param \Drupal\Core\Config\ConfigManagerInterface $configManager
   *   The ConfigManagerInterface service.
   * @param \Drupal\Core\Routing\CurrentRouteMatch $currentRouteMatch
   *   The CurrentRouteMatch service.
   * @param \Drupal\Core\Entity\EntityFieldManager $entityFieldManager
   *   The EntityFieldManager service.
   */
  public function __construct(
      array $configuration,
      $pluginId,
      $pluginDefinition,
      EntityTypeManagerInterface $entityTypeManager,
      ConfigManagerInterface $configManager,
      CurrentRouteMatch $currentRouteMatch,
      EntityFieldManager $entityFieldManager
  ) {
    parent::__construct($configuration, $pluginId, $pluginDefinition);
    $this->entityTypeManager = $entityTypeManager;
    $this->configManager = $configManager;
    $this->currentRouteMatch = $currentRouteMatch;
    $this->entityFieldManager = $entityFieldManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $pluginId, $pluginDefinition) {
    return new static(
        $configuration,
        $pluginId,
        $pluginDefinition,
        $container->get('entity_type.manager'),
        $container->get('config.manager'),
        $container->get('current_route_match'),
        $container->get('entity_field.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'video_component_fields' => [
        'video_url' => '',
        'video' => [],
        'video_field' => [],
        'video_radio_options' => [],
      ],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $formState) {

    // Attach library to the Form.
    $form['#attached']['library'] = ['layout_builder_kit/video-styling'];
    $currentValue = $this->configuration['video_component_fields']['video_radio_options'];
    $nid = '';

    // Get Default radio option.
    if (empty($currentValue)) {
      $currentValue = 0;
    }

    // Get Node Entity from route.
    $currentRoute = $this->currentRouteMatch;
    $sectionStorage = $currentRoute->getParameters()->get('section_storage');
    if ($sectionStorage instanceof OverridesSectionStorage) {
      $context = $sectionStorage->getContext('entity');
      if ($context instanceof EntityContext) {
        $contextData = $context->getContextData();
        if ($contextData instanceof EntityAdapter) {
          $node = $contextData->getEntity();
          if ($node instanceof Node) {
            $nid = $node->id();
          }
        }
      }
    }

    $form['video_radio_options'] = [
      '#type' => 'radios',
      '#default_value' => $currentValue,
      '#options' => [
        0 => $this->t('Video URL'),
        2 => $this->t('Video Fields'),
      ],
      '#id' => 'options_video',
      '#title_display' => $this->t('title'),
      '#weight' => '3',
      '#prefix' => '<div class="video-container"><div class="video-radios--options">',
      '#suffix' => '</div>',

    ];

    $form['video_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Video URL'),
      '#maxlength' => 255,
      '#default_value' => $this->configuration['video_component_fields']['video_url'],
      '#required' => FALSE,
      '#weight' => '4',
      '#prefix' => '<div class="video-fields--options">',
    ];

    $form['video_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Video fields available'),
      '#options' => $this->getVideosEmbedArray($nid),
      '#default_value' => $this->configuration['video_component_fields']['video_field'],
      '#weight' => '5',
      '#suffix' => '</div></div>',
    ];

    $form['#attached']['library'] = ['layout_builder_kit/video-styling'];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $formState) {
    $this->configuration['video_component_fields']['video_radio_options'] = $formState->getValue('video_radio_options');
    $this->configuration['video_component_fields']['video_url'] = $formState->getValue('video_url');
    $this->configuration['video_component_fields']['video_field'] = $formState->getValue('video_field');
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];
    // Add template.
    $build['#theme'] = 'LBKVideo';
    // Attach library.
    $build['#attached']['library'] = ['layout_builder_kit/video-styling'];

    if (!empty($this->configuration['video_component_fields']['video'])) {
      $imageFileId = implode($this->configuration['video_component_fields']['video']);
      $image = File::load($imageFileId);
      if ($image != NULL) {
        $image->setPermanent();
        $image->save();
      }
    }

    // TODO: Render field.
    // View builder.
    $entityType = 'node';
    $node = $this->currentRouteMatch->getParameter('node');

    // Fields to render.
    $fields = [
      $this->configuration['video_component_fields']['video_field'],
    ];

    // Get the Node entity object.
    if ($this->currentRouteMatch->getParameter('node')) {
      $node = $this->currentRouteMatch->getParameter('node');
      if ($node instanceof Node) {
        $nodeEntity = Node::load($node->id());
      }
    }
    else {
      $currentRoute = $this->currentRouteMatch;
      $sectionStorage = $currentRoute->getParameters()->get('section_storage');
      if ($sectionStorage instanceof OverridesSectionStorage) {
        $context = $sectionStorage->getContext('entity');
        if ($context instanceof EntityContext) {
          $contextData = $context->getContextData();
          if ($contextData instanceof EntityAdapter) {
            $node = $contextData->getEntity();
            if ($node instanceof Node) {
              $nodeEntity = Node::load($node->id());
            }
          }
        }
      }
    }

    // Get the View for this Node entity object.
    $viewBuilder = $this->entityTypeManager->getViewBuilder($entityType);

    // Build the Node array to render.
    $fieldOutput = [];
    foreach ($fields as $fieldName) {
      if (!empty($fieldName)) {
        if (isset($nodeEntity)) {
          if ($nodeEntity->hasField($fieldName) && $nodeEntity->access('view')) {
            $value = $nodeEntity->get($fieldName);
            $fieldOutput = $viewBuilder->viewField($value, ["label" => "hidden"]);
            $fieldOutput['#cache']['tags'] = $nodeEntity->getCacheTags();
          }
        }
      }
    }

    //$this->configuration['video_component_fields']['title'] = $formState->getValue('title');
    $build['#title'] = $this->configuration['title'];

    //$build['#display_title'] = $this->configuration['video_component_fields']['display_title'];
    $build['#display_title'] = $this->configuration['display_title'];

    $build['#video_radio_options'] = $this->configuration['video_component_fields']['video_radio_options'];
    $build['#video_url'] = $this->getVideoEmbedUrl($this->configuration['video_component_fields']['video_url']);
    $build['#video_field'] = $fieldOutput;
    $build['#classes'] = $this->configuration['classes'];

    return $build;
  }

  /**
   * Get fields from Content Type.
   *
   * @param string $contentType
   *   The Content Type.
   *
   * @return array
   *   Return an array of fields.
   */
  public function getContentTypeFields($contentType) {

    $fields = [];

    if (!empty($contentType)) {
      $fields = array_filter(
          $this->entityFieldManager->getFieldDefinitions('node', $contentType), function ($fieldDefinition) {
            return $fieldDefinition instanceof FieldConfigInterface;
          }
      );
    }

    return $fields;
  }

  /**
   * Get Videos Embed in array.
   *
   * @param int $nid
   *   The nid value.
   *
   * @return array
   *   The video array.
   */
  public function getVideosEmbedArray($nid) {

    $fieldsArray = [];
    $node = Node::load($nid);

    if (!empty($nid)) {
      if ($node instanceof Node) {
        $bundle = $node->bundle();
        $fieldsNode = $this->getContentTypeFields($bundle);

        $fieldsType = [];
        foreach ($fieldsNode as $field) {
          if ($field instanceof FieldConfig) {

            $fieldsType[$field->getName()] = $field->getType();
            unset($fieldsType['layout_builder__layout']);

            // Filtering by field type.
            switch ($field->getType()) {
              case "string":
                $fieldsArray[$field->getName()] = $field->getLabel();
                break;

              case "video_embed_field":
                $fieldsArray[$field->getName()] = $field->getLabel();
                break;

              case "link":
                $fieldsArray[$field->getName()] = $field->getLabel();
                break;

              case "file":
                $fieldsArray[$field->getName()] = $field->getLabel();
                break;
            }
          }
        }
      }
    }

    return $fieldsArray;
  }

  /**
   * Get Video Embed Url.
   *
   * @param string $videoUrl
   *   The Video URL.
   *
   * @return string
   *   The URL.
   */
  public function getVideoEmbedUrl($videoUrl) {
    $finalVideoUrl = '';

    if (strpos($videoUrl, 'vimeo.com/') !== FALSE) {

      // It is Vimeo video.
      $videoId = explode("vimeo.com/", $videoUrl)[1];
      if (strpos($videoId, '&') !== FALSE) {
        $videoId = explode("&", $videoId)[0];
      }
      $finalVideoUrl .= 'https://player.vimeo.com/video/' . $videoId;
    }
    elseif (strpos($videoUrl, 'youtube.com/watch') !== FALSE) {

      // It is Youtube video.
      $videoId = explode("v=", $videoUrl)[1];
      if (strpos($videoId, '&') !== FALSE) {
        $videoId = explode("&", $videoId)[0];
      }
      $finalVideoUrl .= 'https://www.youtube.com/embed/' . $videoId;
    }
    elseif (strpos($videoUrl, 'youtu.be/') !== FALSE) {

      // It is Youtube video.
      $videoId = explode("youtu.be/", $videoUrl)[1];
      if (strpos($videoId, '&') !== FALSE) {
        $videoId = explode("&", $videoId)[0];
      }
      $finalVideoUrl .= 'https://www.youtube.com/embed/' . $videoId;
    }
    elseif (strpos($videoUrl, 'youtube.com/embed') !== FALSE) {

      // It is a valid Youtube embed video.
      $finalVideoUrl .= $videoUrl;

    }

    return $finalVideoUrl;
  }

}
